
#ifndef BSPLINE2D_H 
#define BSPLINE2D_H

#include <sampledfunction.h>
#include <matvect.h>

class EqualStepBSpline2D {
private :
  
  double xmin;
  double xmax; 
  
  double ymin;
  double ymax; 
  
  size_t nx;
  size_t ny;
  
 public : // for test
  double xscale;
  double xoffset;
  double yscale;
  double yoffset;

  void SetScales() {
    
    if(xmax>xmin)
      xscale = (nx-2)/(xmax-xmin);
    xoffset = -xmin*xscale+2;
    if(ymax>ymin)
      yscale = (ny-2)/(ymax-ymin);
    yoffset = -ymin*yscale+2;
  }
  
public :
  Vect params;
  EqualStepBSpline2D(){   
    xmin=0;
    xmax=1;
    xscale=1;
    yoffset=0;
    ymin=0;
    ymax=1;
    yscale=1;
    yoffset=0;
  }
  void SetNPar(int i_nx,int i_ny) {
    nx = i_nx;
    ny = i_ny;
    SetScales();
    int n = nx*ny;
    if(n != int(params.size()))
      params.allocate(n);
  }
  void SetRange(const double& i_xmin, const double& i_xmax, const double& i_ymin, const double& i_ymax) {
    xmin = i_xmin;
    xmax = i_xmax;
    ymin = i_ymin;
    ymax = i_ymax;
    SetScales();
  }
  
  EqualStepBSpline2D(const double& i_xmin, const double& i_xmax, int i_nx,
	    const double& i_ymin, const double& i_ymax, int i_ny
	    )
  {
    SetRange(i_xmin,i_xmax,i_ymin,i_ymax);
    SetNPar(i_nx,i_ny);
  } 
  virtual double Value(const double& x, const double& y, bool zero_outside_range = false) const;
  double Xmin() const{return xmin;}
  double Xmax() const{return xmax;}
  double Ymin() const{return ymin;}
  double Ymax() const{return ymax;}
  int Nx() const{return nx;}
  int Ny() const{return ny;}
  virtual bool Fit(size_t i_nx, const double*x, size_t i_ny, const double* y,  double** f, bool adapt_npars=true);
  virtual bool Fit(Grid2DFunction &func);

  EqualStepBSpline2D(Grid2DFunction &func){ 
    if( ! Fit(func)) abort();
  }  
  
};

class BSpline2D : public EqualStepBSpline2D {

 private : 
  
  Vect original_x_nodes;
  Vect remapped_x_nodes;
  Vect original_y_nodes;
  Vect remapped_y_nodes;
  

public :
  
  BSpline2D() {EqualStepBSpline2D::EqualStepBSpline2D();};
  
  virtual double Value(const double& x, const double& y, bool zero_outside_range = false) const;
  double RemappedX(const double& x) const;
  double RemappedY(const double& y) const;
  virtual bool Fit(size_t nx, const double*x, size_t ny, const double* y,  double** f, bool adapt_npar=true);
  virtual bool Fit(Grid2DFunction &func);
  BSpline2D(Grid2DFunction &func){ 
    if( ! Fit(func)) abort();
  }  
};

#endif
