#include <iostream>
#include "corrector.h"


using namespace std;

  // parameters
  // 0 : phase
  // 1 : stretch
  // 2 : lambda
  // 3 : colorbv

#include <cstdlib> // for abort()

Corrector::Corrector(int dimension_i, int* degrees_i, 
		     double* XMin, double* XMax, double X1Cut) :
  stretch_pol(dimension_i-1,degrees_i,XMin,XMax,X1Cut),
  color_pol(2,degrees_i+2,XMin+2,XMax+2,100000.)
{
  
  
  if(dimension_i!=4) {
    cerr << "cannot handle dimension!=4 " << endl;
    abort();
  }
  
#ifdef DEBUG  
  for(int i=0;i<stretch_pol.Dimension();i++)
    cout << "degree " << i << " " << stretch_pol.GetDegree(i) << endl;
  for(int i=0;i<color_pol.Dimension();i++)
    cout << "degree " << i << " " << color_pol.GetDegree(i) << endl;
  
  for(int i=0;i<Dimension();i++)
    cout << "degree " << i << " " << GetDegree(i) << endl;
#endif
  
}


Corrector::Corrector(const LegSplitN &stretch_pol_in,const LegSplitN &color_pol_in) { 
  stretch_pol=stretch_pol_in;
  color_pol=color_pol_in;
}

Corrector::Corrector(const std::string &FileSuffix) :
  stretch_pol(FileSuffix+"_stretch"),
  color_pol(FileSuffix+"_color")
{
}

Corrector::Corrector(const std::string &FileName_stretch,const std::string &FileName_color) :
  stretch_pol(FileName_stretch),
  color_pol(FileName_color)
{
}

bool Corrector::Write(const std::string &FileSuffix) const {
  if ( ! stretch_pol.Write(FileSuffix+"_stretch") ) return false;
  return color_pol.Write(FileSuffix+"_color");
}

double Corrector::Value(const double *X) const {
  return stretch_pol.Value(X)+color_pol.Value(X+2);
}

Corrector Corrector::operator +(const Corrector& Right) const {
  Corrector res(*this);
  res += Right;
  return res;
}

void Corrector::operator +=(const Corrector& Right) {
  stretch_pol+=Right.stretch_pol;
  color_pol+=Right.color_pol;
}

int Corrector::CoeffSize() const {
  return stretch_pol.CoeffSize() + color_pol.CoeffSize();
}

void Corrector::Monomials(const double* X, double *Monom) const {
  stretch_pol.Monomials(X,Monom);
  color_pol.Monomials(X+2,Monom+stretch_pol.CoeffSize());
}

void Corrector::DMonomialsDXN(int N, const double* X, double * DMonomDxN) const {
  stretch_pol.DMonomialsDXN(N,X,DMonomDxN);
  if (N>=2) {
    stretch_pol.DMonomialsDXN(N-2,X+2,DMonomDxN+stretch_pol.CoeffSize());
  }else{
    for(int i=stretch_pol.CoeffSize();i<CoeffSize();i++) {
      DMonomDxN[i]=0;
    }
  }
}

int Corrector::GetDegree(int d) const {
  if(d<stretch_pol.Dimension())
    return stretch_pol.GetDegree(d);
  else
    return color_pol.GetDegree(d-2);
}

bool Corrector::IsValidValue(int d,double val) const {
  if(d<stretch_pol.Dimension())
    return stretch_pol.IsValidValue(d,val);
  else
    return color_pol.IsValidValue(d-2,val);
}

void Corrector::SetCoeffs(const double *Val)  {
  stretch_pol.SetCoeffs(Val);
  color_pol.SetCoeffs(Val+stretch_pol.CoeffSize());  
}
