#include <cmath>

#include "cosmology.h"
#include "cosmology_prior.h"
#include "matvect.h"

static double sq(const double &x) {return x*x;}

// priors
//===============================================================

Mat Eisenstein2005_Prior::Weight(const Cosmology *Cosmo) const
{
  Mat W(1,1);
  W(0,0)= 1./sq(0.017);
  return W;
}
//! the equation 4 and 5 of astro-ph 0501117  
Vect Eisenstein2005_Prior::Residual(const Cosmology *Cosmo) const
{
  double z1 = 0.35;
  double A=sqrt(Cosmo->Omegam())*pow(Cosmo->E(z1),-1./3.)*
    pow(Cosmo->Dm(z1)/z1,2./3.);
  Vect R(1);
  R(0) = A-0.469;
  return R;
}

Mat OmegaM_Prior::Weight(const Cosmology *Cosmo) const
{
  Mat W(1,1);
  W(0,0)= 1./sq(omega_m_error);
  return W;
}

Vect OmegaM_Prior::Residual(const Cosmology *Cosmo) const
{
  Vect R(1);
  R(0) = Cosmo->Omegam()-omega_m_val;
  return R;
}

double Prior::Chi2(const Cosmology *Cosmo) const
{
  Vect R = Residual(Cosmo);
  return scalar_product(R,Weight(),R);
}



