#ifndef COSMOLOGY_PRIOR_H
#define COSMOLOGY_PRIOR_H


#include <string>

#include "countedref.h"

class Mat;
class Vect;
class Cosmology;

class Prior : public RefCount {

  public :
    Prior() {name="unknown";};
  virtual ~Prior(){};
    std::string name;
    virtual Mat Weight(const Cosmology *Cosmo=0) const = 0;
    virtual Vect Residual(const Cosmology *Cosmo) const = 0;
    virtual double Chi2(const Cosmology *Cosmo) const;
};

class Eisenstein2005_Prior : public Prior {
  public :
    Eisenstein2005_Prior() : Prior() {name="Eisenstein2005";};
  
  
  Mat Weight(const Cosmology *Cosmo=0) const;
  Vect Residual(const Cosmology *Cosmo) const;
};

class OmegaM_Prior : public Prior {
  private :
    double omega_m_val;
  double omega_m_error;

  public :
    OmegaM_Prior(const double& om, const double& err) : Prior() {
    name="Om";
    omega_m_val = om;
    omega_m_error = err;

  };
  
  Mat Weight(const Cosmology *Cosmo=0) const;
  Vect Residual(const Cosmology *Cosmo) const;
  
};

#endif 
