#ifndef COVARIANCEMODEL__H
#define COVARIANCEMODEL__H

#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include "wavelength.h"
#include "poly1.h"
//#include "sn4pca.h"


/*
  covariance(data1,data2) = (lambda-l_B)^2*(lambda-l_V)^2*P1(lambda)*exp(-|phase1-phase2|/P2(lambda)) *model1*model2
*/
class CovarianceModel {
public :
  Poly1 P1;
  Poly1 P2;
  CovarianceModel(size_t n_p1=1,size_t n_p2=1) {
    P1.SetRange(3000.,7000.);
    P2.SetRange(3000.,7000.);
    P1.SetDeg(n_p1-1);
    P2.SetDeg(n_p2-1);
  }
  double Cov(const double& phase1, const double& phase2, const double& lambda) const {

    double rl  = (lambda-B_WAVELENGTH)/(V_WAVELENGTH-B_WAVELENGTH);
    double rlu = (U_WAVELENGTH-B_WAVELENGTH)/(V_WAVELENGTH-B_WAVELENGTH);
    // rl = 0 in B
    //    = 1 in V
    double prefact = (rl*(rl-1.))/(rlu*(rlu-1.));
    prefact *= 1.e-3*prefact;
    
    // prefact is 0 in B and V and 0.001 in U
    
    double logval = 0;
    if(P2.NPar()==0) {
      // P1 is the polynomial of ColorLawError
      // ColorLawError.value() is a variance
      return prefact*exp(P1.Value(rl));
    }else{
      double pval = P2.Value(rl);
      if(pval<2.) pval=2.; // minimum is 2 days     
      logval = P1.Value(rl)-fabs(phase1-phase2)/pval;
      if(logval>50) logval=50;
    }
    return prefact*exp(logval);
  }
 
  void Write(std::ostream &file) const {
    P1.Write(file);
    P2.Write(file);   
  }
  void Write(const std::string &FileName) const {
    std::ofstream stream(FileName.c_str());
    Write(stream);
    stream.close();
  }
  bool Read(std::ifstream &file, const std::string &FileName) {
    std::cout << "reading covariance model in " << FileName << std::endl;
    if(! P1.Read(file)) return false;
    return P2.Read(file);
  }
  bool Read(const std::string &FileName) {
    std::ifstream stream(FileName.c_str());
    bool status = Read(stream,FileName);
    stream.close();
    return status;
  }
};

#endif
