#ifndef FILTER__H
#define FILTER__H


#include "sampledfunction.h"
#include "globalval.h"


//! just a wrapper for a Filter defined through a table of transmissions. Wavelengthes expected in Angstroms.
class Filter : public General1DFunction {
  
 private:
  string filterName, instrumentName;
  GlobalVal glob;
  
 public : 
  //! - 
  Filter(const string &FileName, const string &FilterName, 
	 const string &InstrumentName) : 
    General1DFunction(FileName), 
    filterName(FilterName), 
    instrumentName(InstrumentName), 
    glob(FileName)
      {};

 
    //!-
  Filter(const General1DFunction &func,const string &FilterName, 
	 const string &InstrumentName)
    : General1DFunction(func), 
      filterName(FilterName), 
      instrumentName(InstrumentName) 
	{};
  //! - 
      /*
	Filter(const string &FilterName, const string &InstrumentName) :
	filterName(FilterName), instrumentName(InstrumentName) {};
      */
  //! default constructor
  Filter() : filterName("NoName"), instrumentName("?") {};

  //! names that combine instrument name and filter name
  /*! do not change this routine without updating accordingly 
    InstrumentNameFromFilterName */
  string Name() const { return instrumentName+"::"+filterName;};

  //! -
  const string& InstrumentName() const { return instrumentName;}

  //! -
  const string& Band() const {return filterName;};

  //! -
  const GlobalVal &Globals() const { return glob;}
  
  //! - 
  friend ostream& operator << (ostream &stream, const Filter& Filter);
  
  //! just the mean 
  double CentralWaveLength() const;

  //! 
  bool Write(const string &FileName) const;


};

//! extract the instrument name from Filter::Name() output
/*! use Filter::InstrumentName() if you hold an actual filter, rather than just its name */
string InstrumentNameFromFilterName(const std::string &FilterName);

#endif /* FILTER__H */
