#include <stdio.h>
#include <iostream>
#include <fstream>

#include "param.h"
#include "fittedsn.h"
#include "fileutils.h"
#include "fatalerror.h"

// for the definition of the datasets
#include "malmquist_bias.h"

#define DEBUG



FittedSn::FittedSn () {
  Name = "UNKNOWN";
  Set = Other;
  RedShift = 0;
  Gal_Type =-200;
  group    = 0;
  mag      = 0;
  shape    = 0;
  color    = 0;
  model    = UNKNOWN;
#ifdef WITH_HOST_EXTINCTION
  has_extinction_measurement=false;
#endif
#ifdef WITH_UV_FIT
  has_uv_fit=false;
#endif

}

FittedSn::~FittedSn () {
}



//======================================



void AllSn::ReadDictFile(const string& dictfilename) {
  
  
  DictFile file(dictfilename);
  int group = 0;
  if(size()!=0)
    group = back()->GetGroup()+1;
  
  
  for (DictFileCIterator line = file.begin(); line != file.end();  ++line) {
    
    FittedSn *sn = new FittedSn();
    
    sn->SetGroup(group);
    sn->RedShift = line->Value("z");

    if (line->HasKey("gtype"))
      sn->Gal_Type = line->Value("gtype");
    if(line->HasKey("name"))
      sn->Name = string(line->Value("name"));
    
    if(line->HasKey("set"))
      sn->Set = dataset_index(string(line->Value("set")));
    else
      sn->Set = Other;
    
    sn->mag     = sn->params.LocateOrAddNonConstParam("mag");
    sn->shape   = sn->params.LocateOrAddNonConstParam("shape");
    sn->color   = sn->params.LocateOrAddNonConstParam("color");
    
    string mag_key   = "msb";
    string color_key = "c";
    
    string shape_key;
    
    if (line->HasKey("s") ) { // has stretch, so it's a salt1 fit
      shape_key = "s";
      sn->model = SALT1;
    }else{
      if (line->HasKey("x1") ) { // has X1, so it's a salt2 fit
	shape_key = "x1";
	sn->model = SALT2;
      }else{
	cout << "line has to 's' nor 'x1' so i dont know which model to use" << endl;
	exit(12);
      }
    }
    
    sn->mag->val = line->Value(mag_key);
    sn->mag->err = line->Value(mag_key+"e");
    sn->shape->val = line->Value(shape_key);
    sn->shape->err = line->Value(shape_key+"e");
    sn->color->val = line->Value(color_key);
    sn->color->err = line->Value(color_key+"e");
    
    
    if(sn->model == SALT1) { // or sifto actually
      sn->shape->val -= 1; // stretch-1
    }
    
    
    // covariances
    if(line->HasKey(mag_key+shape_key+"_c"))
      sn->params.SetCovariance("mag","shape",line->Value(mag_key+shape_key+"_c"));
    else // other ordering
      sn->params.SetCovariance("mag","shape",line->Value(shape_key+mag_key+"_c"));
    
    if(line->HasKey(mag_key+color_key+"_c"))
      sn->params.SetCovariance("mag","color", line->Value(mag_key+color_key+"_c"));
    else // other ordering
      sn->params.SetCovariance("mag","color", line->Value(color_key+mag_key+"_c"));
    
    if(line->HasKey(shape_key+color_key+"_c"))
      sn->params.SetCovariance("shape","color",line->Value(shape_key+color_key+"_c"));
    else // other ordering
      sn->params.SetCovariance("shape","color",line->Value(color_key+shape_key+"_c"));
    
    
    
#ifdef WITH_HOST_EXTINCTION
    // new check whether there is an extinction measurement
    if(line->HasKey("ebv")) {
      sn->has_extinction_measurement = true;
      
      has_extinction_measurement = true;
      
      sn->ebv   = sn->params.LocateOrAddNonConstParam("EBV");
      sn->ebv->val = line->Value("ebv");
      sn->ebv->err = line->Value("ebve");
      
      cout << "warning : ignore covariance between EBV and other params !" << endl;
#warning ignore covariance between EBV and other params
    }else{
      if(has_extinction_measurement) {
	cerr << "sn has no extinction (ebv key) but some other sne in file have" << endl;
	exit(12);
      }
    }
#endif
#ifdef WITH_UV_FIT	
    if(line->HasKey("uv")) {
      sn->has_uv_fit = true;	  
      has_uv_fit = true;
      
      sn->uv   = sn->params.LocateOrAddNonConstParam("UV");
      sn->uv->val = line->Value("uv");
      sn->uv->err = line->Value("uve");
      
      cout << "warning : ignore covariance between UV and other params !" << endl;
#warning ignore covariance between UV and other params
      
    }else{
      if(has_uv_fit) {
	cerr << "sn has no UV (uv key) but some other sne in file have" << endl;
	exit(12);
      }
    }
#endif
    
    
    if(line->HasKey("gtype")) {
      int gtype = int(line->Value("gtype"));
      if(gtype>=0) { // means we know something about the type of this sn
	sn->Gal_Type = gtype;
	// check if we have this type in the SN sample
	bool i_have_it = false;
	for(size_t i=0;i<galaxy_types.size();++i) {
	  if(galaxy_types[i]==gtype) {
	    i_have_it = true;
	    break;
	  }
	}
	if( ! i_have_it) galaxy_types.push_back(gtype);
	  
      }
    }

    if(model==UNKNOWN) {
      model = sn->model;
    }else {
      if( model != sn->model ) {
	cerr << "incompatible SN models for this AllSn dataset" << endl;
	exit(12);
      }
    }
    push_back(sn);
  }
}
