#ifndef GALAXY_TEMPLATE_H
#define GALAXY_TEMPLATE_H

#include <string>
#include <map>
#include "sampledfunction.h"
#include "matvect.h"

// when sfr = 0
#define SSFR_MIN_VALUE -100
// what does zero means :
#define U_SSFR_MIN_VALUE 1.e-20
// when no value is available
#define SSFR_BAD_VALUE -1000



class GalaxyTemplate {
  
  
  
 public :
  
    //x = age for a Pegase template, x=tau or mean star pop age  for the exponential ... templates. Here x is called "the type index".
    //y=lambda
  Grid2DFunction continuum; 
  const double *GalaxyTemplate_Indexation_vals() const { return continuum.xvals();}
  const int GalaxyTemplate_Indexation_N() const { return continuum.NstepX();}
  
  bool HasLineSpectrum ;
  Grid2DFunction lines;
  Vect lines_fvals_buffer;
  void LoadTemplates(const string& cardfilename="");
  
  General1DFunction SFR ;
  General1DFunction Stellar_Mass ;
  General1DFunction Metallicity ;
 
  bool has_sfr, has_stellar_mass, has_metallicity ;
  bool Has_Metallicity()const {return(has_metallicity);}
  bool Has_SFR()const {return(has_sfr);}
  bool Has_Stellar_Mass() const {return(has_stellar_mass);}
  double sSFR(double age) const ;

  GalaxyTemplate(const string& cardfilename="");


  //beware (y,x) !!
  double Value(const double& wavelength, const double& type_index) const;
  Grid2DXSlice ContinuumSpectrum(const double& Type) const;
  General1DFunction LineSpectrum(const double& Type) const;
  double MinType() const {return continuum.Xmin();};
  double MaxType() const {return continuum.Xmax();};
  double MinWavelength() const {return continuum.Ymin();};
  double MaxWavelength() const {return continuum.Ymax();};
  
};

#endif // GALAXY_TEMPLATE_H
