#ifndef HUBBLEFITTER_H
#define HUBBLEFITTER_H

#include <iostream>
#include <string>

#include "param.h"
#include "fittedsn.h"
#include "hubblemodel.h"
#include "matvect.h"

class Prior;

class HubbleFitter {

 private:

  const AllSn &allsn;
  HubbleModel &hubblemodel;
  vector < CountedRef<Prior> > Priors;
  
 public:

  HubbleFitter(const AllSn &AllSn, HubbleModel &Model);
  ~HubbleFitter() {};

  HubbleModel *GetHubbleModel() {return &hubblemodel;};
  
  int Ndeg_of_Freedom();
  double TheChi2();
  double MinimizeTheChi2_WithIntrisicDispersion(const double& intrinsic_dispersion, const bool intrinsic_dispersion_is_fixed);
  void InitTheFit();
  void MinimizeTheChi2(bool initfit=true);
  Mat GetCovMat();
  //void ReloadData(const AllSn &AllSn) {allsn=AllSn;};
  
  void dump(std::ostream & stream = std::cout) const{
    hubblemodel.hubbleparams.dump(stream);
  }
  friend ostream& operator << (ostream &stream, const HubbleFitter &hubble)
    { hubble.dump(stream);return stream;}

  void AddPrior(CountedRef<Prior> p) {
    Priors.push_back(p);
  }

};
// SMALL APPLICATIONS USEFUL ELSEWHERE

struct box {
  HubbleFitter *hubblefitter;
  int ndf;
};
double chi2_for_intrinsic_dispersion(const double& intrinsic_dispersion, const void* context);


#endif /* HUBBLEFITTER_H */
