#ifndef LCFIT__H
#define LCFIT__H

#include <iostream>
#include <string>
#include <map>
#include <vector>

#include "measureddata.h"
#include "model2fit.h"
#include "param.h"



/*! a class for fitting light curve models to data with Minuit 
 *
 * all pertinent info is in measuredData and model (which is a virtual class)
 * contains all the minuit handling
 */
class LcFit
{
  private :

    
    const MeasuredData &measuredData;
    Model2Fit &model;
    
    const double z;
    double chi2Val;
    double chi2Valpdf;
    double phasemin;
    double phasemax;
    int noutliers;
    public :
      
      LcFit(const MeasuredData &Data, Model2Fit& Model);
    
    Model2Fit* GetModel() {return &model;};
    
    
    
    int NPar () const {return model.params.size();}
    
    /** chi2 of a single band (data points from MeasuredData::LightCurve(Band)) */
    double Chi2(const size_t Band) const;
    double Chi2Without(const size_t Band, MeasuredPointCIterator i) const;
    
    double Chi2() const;
    
    void InitFit();
    
    bool Minimize();

    double ModelFlux(const MeasuredPoint* datapoint) const;
    
    void dump(std::ostream & stream = std::cout) const{
      stream << model.ModelName() << endl;
      model.params.dump(stream);
    }
    friend ostream& operator << (ostream &stream, const LcFit &lc)
      { lc.dump(stream); return stream;}
    

    virtual ~LcFit() {}
    void SetPhaseRange(double pmin, double pmax) {phasemin=pmin;phasemax=pmax;};

};



#endif /* LCFIT__H */
