#ifndef LCTEMPLATE__H
#define LCTEMPLATE__H


#include <string>
#include <map>

#include "param.h"
#include "model2fit.h"
#include "templatecurves.h"
#include "sampledfunction.h"
#include "measureddata.h"



class LcTemplate : public Model2Fit 
{
  private :
    TemplateCurves templateCurves; 
  map<const string,BandParams*> Params4Band; // map of parameters for one fitted band, internal storage  
  map<const Filter*,string> BandOfFit4ObserverFilter; // map of associations between ObserverFilter and fitted bands
  map<const Filter*,BandParams*> Params4ObserverFilter; // shortcut to go faster in the fit
  bool fitsingleband;
  bool forcepositiveEBV;
  string singleband;
  double GetMag(double phase,const string &magsys);
  
  public :
    // create minimal number of parameters needed for the fit
    //void CreateParams(); // do we need this ???
  void CreateNeededParams(const MeasuredData* data);
  void SetRedshift(double zz) {z=zz;BandOfFit4ObserverFilter.clear();Params4ObserverFilter.clear();};
  void ReadParams(const string& filename,bool createifmissing = true);
  bool CanComputeFluxFor(const Filter &observerfilter) const;
  
  map<const string,BandParams*> & GetParams4Band() {return Params4Band;};
  

  // dynamic association of ObserverFilters to parameters
  BandParams* GetParams4ObserverFilter(const Filter* filter, bool computeifnecessary=true);
  void FitSingleBand(const string &band);
  void KeepOnlyNBands(int n,const MeasuredData* data,vector<string> &data2remove);
  void ForcePositiveEBV(bool yes = "true") { forcepositiveEBV=yes;};
  const TemplateCurves& GetTemplateCurves() const {return templateCurves;};
  LcParam GetRestFrameMagAtPhase(double phase, const string &magsys= "VEGA") ;
  LcParam GetUnextinctedRestFrameMagAtPhase(double phase, const string &magsys= "VEGA") ;
  
  double GetRestFrameMagAtPhase(string const& band,
				double phase,
				string const& magsys="VEGA");
  
  double GetUnextinctedRestFrameMagAtPhase(string const& band,
					   double phase, 
					   const string &magsys= "VEGA");
  
  LcTemplate();
  LcTemplate(const string &ModelFileName);
  
  double Flux(const Filter* filter,
	      const double observerday);
  void InitParams(const MeasuredData* data);
  string ModelName() const {return "LcTemplate";};
  double GetPhase(double day) const;
};


#endif /* MODEL2FIT__H */
