#ifndef LEGENDRE__H
#define LEGENDRE__H



#include <string>
#include <vector>


using namespace std;


//! 
class LegSplit3
{
  vector<double> coeffs;
  int degx, degy, degz;
  double ax,bx, ax2, bx2, ay,by, az,bz;
  int coeffSize;

  public :
    LegSplit3(int Dx, int Dy, int Dz,
	      double XMin, double XMax,  double XCut, 
	       double YMin, double YMax, 
	       double ZMin, double ZMax);
  
  LegSplit3(const string &FileName){Read(FileName);};
  LegSplit3(const LegSplit3& Right);
  bool Write(const string &FileName);
  
  int CoeffSize() const;

  void Monomials(const double &X, const double &Y, 
		 const double &Z, double *Monom) const;
  
  void DMonomials(const double &X,
		  const double &Y,
		  const double &Z,
		  double* DMonomDx,
		  double* DMonomDy,
		  double* DMonomDz) const;

  

  double Value(const double &X, const double &Y, const double &Z) const;
  
  // LegSplit3 & operator =(const LegSplit3& Right); // trivial
  LegSplit3 operator +(const LegSplit3& Right) const;
  void operator +=(const LegSplit3& Right);


  bool Write(const string &FileName) const;
  bool Read(const string &FileName);

  void SetCoeffs(const double *Val);
  bool SameDegrees(const LegSplit3& Right) const;
};
  
  
//! 
class LegSplitN
{
  vector<double> coeffs;
  int *degrees;
  double *a;
  double *b;
  double *buffer;
  double ax2,bx2;
  
  
  int coeffSize;
  int dimension;
  
  void Zero();
  void Free();
  void Allocate(int dim,int *degrees_i);
  
  public :
    
    LegSplitN(int dimension_i, int* degrees_i,
	      double* XMin, double* XMax, double X1Cut); 
  
  LegSplitN(const string &FileName);
  LegSplitN(const LegSplitN& Right);
  LegSplitN();
  
  virtual ~LegSplitN();
  
  bool Write(const string &FileName) const;
  
  int Dimension() const {return dimension;};
  int CoeffSize() const;
  int GetDegree(int d) const;
  bool IsValidValue(int d,double val) const;
  void Monomials(const double* X, double *Monom) const;
  
  void DMonomialsDXN(int N, const double* X, double * DMonomDxN) const;
  void DMonomials(const double *X, double** DMonomDx) const;

  double Value(const double *X) const;
  double Value(const double X1, const double X2=0, const double X3=0, const double X4=0) const;
  double Derivative(int d, const double *X) const;
  void Scale(double s);
  
  LegSplitN operator +(const LegSplitN& Right) const;
  void operator +=(const LegSplitN& Right);


  
  bool Read(const string &FileName);

  void SetCoeffs(const double *Val);
  bool SameDegrees(const LegSplitN& Right) const;
};
  
  
#endif /* LEGENDRE__H */
