#include <iostream>
#include <cstdlib> // for exit
#include "malmquist_bias.h"

using namespace std;

int dataset_index(const std::string& datasetname) {
  if(datasetname=="SNLS") return SNLS;
  if(datasetname=="Hamuy1996") return Hamuy1996;
  if(datasetname=="Riess1999") return Riess1999;
  if(datasetname=="Jha2006") return Jha2006;
  if(datasetname=="Hicken2009") return Hicken2009;
  if(datasetname=="SDSS") return SDSS;
  if(datasetname=="Riess07") return Riess2007;
  if(datasetname=="Other") return Other;
  if(datasetname=="other") return Other;
  cout << "WARNING malmquist_bias : no malmquist bias for unhandled dataset " << datasetname << endl;
  return Other;
}

static double sqr(const double& x) {return x*x;}

double malmquist_bias(const double &z, int dataset) {

  //cout << " malmquist_bias ! " << endl;
  // if(z<0.14) return -0.017; // as in Astier 2006
  
  switch(dataset) {
  case SNLS :
    if(z>0.55) {
      //return 0;
      return -0.025*sqr((z-0.55)/(1-0.55)); // fit du bias tres tres bizarre de Alex
      //return -0.05*sqr((z-0.6)/(1-0.6)); // same as Astier06
    }else 
      return 0.;
    
  case Riess2007 : return 0;
  case Riess1999 :  return -0.02; // according to Alex&Mark
  case Hamuy1996 :  return -0.02; // according to Alex&Mark
  case Jha2006 :  return 0.; // according to Alex&Mark
  case Hicken2009 :  return -0.017; // return -0.0324; // from Alex (mail 2009-07-10) 
  case Other :  return -0.017; //as in Astier 2006, need to be revised
  case SDSS :  return 0; // unknown
    
    
  default:
    cout << "WARNING malmquist_bias : no malmquist bias for unhandled dataset number " << dataset << endl;
    abort();
  }
  
  return 0; // should never come here
}
