#ifndef PARAM__H
#define PARAM__H

#include <iostream>
#include <vector>
#include <iostream>
#include <string>
#include <map>

using namespace std;

//! Named parameters (i.e. what estimation theory call parameters).
class LcParam
{
  public :
    
    double val, err, step, minv, maxv;
  bool isfixed;
  std::string name;
  LcParam() : val(0), err(-1), isfixed(false), name("UNKNOWN") {};
  LcParam(const string &Name) : val(0), err(-1), isfixed(false), name(Name) {};

  std::string Name() const { return name;}
  bool FindInName(const string &Pattern )
    { return (name.find(Pattern) != name.npos);}
  void dump(ostream &stream = cout) const;
  friend ostream& operator << (ostream &stream, const LcParam &p)
    { p.dump(stream); return stream;}
};


//! the ensemble of all parameters of the fit
/* Implementation note :

   This class cannot be a vector of objects, since DataPoints hold a
   pointer to their param set (i.e. the ones that refer to their own
   band via BandParam's), and vector's change location of already
   allocated objects when calling push_back. This is why it is a
   vector of pointers.  A vector of const pointers (LcParam const *)
   does not work, nor LcParam const &.
*/

class FitParams : public vector<LcParam *>
{
    // we should add here a covariance matrix of parameters
  // DONE !
// the covariance matrix is "indexed" by parameter names, as for parameters
  map<string,double> covMat; 
  string name;

    public :

  FitParams(const string &Name) : name(Name) {};
  FitParams() : name("NoName") {};
  void SetName(const string &Name) { name=Name;}

    FitParams(const FitParams &); 

    const LcParam* LocateParam(const string &ParamName) const;
    LcParam* LocateParam(const string &ParamName);
    const LcParam* LocateOrAddParam(const string &ParamName);
    LcParam* LocateOrAddNonConstParam(const string &ParamName);
    bool DeleteParamIfExists(const string &ParamName);


    size_t VariableParamCount() const;

    //    iterator LocateParam(const string &Name);
    void dump(ostream &stream = cout, const string &Message = "") const;
    void dump_model(ostream &stream = cout, const string &Message = "") const;
    void dump_correlation(ostream &stream, const string &Message = "") const;
    
    friend ostream& operator << (ostream &stream, const FitParams &p)
      { p.dump(stream); return stream;}

    /*! if FitParam has a name, read the block with the same name else reads the first block and assigns name */
    void read(const std::string &FileName);
    void read(istream &S);



  double GetCovariance(const string &Name1, const string &Name2) const;
  void SetCovariance(const string &Name1, const string &Name2, const double &Val);

    
    ~FitParams()
	{ for (FitParams::iterator i=begin(); i != end(); ++i) delete *i;}
         
    void operator = (const FitParams &);
};

typedef FitParams::iterator ParamIterator;
typedef FitParams::const_iterator ParamCIterator;



//! the parameters relevant for one spectral band. Wether they are shared or not between several bands is not known by this class.
class  BandParams
{
  const LcParam *dayMax, *stretch, *dFlux, *ext;
  
  public :
    BandParams(const LcParam &dmax, const LcParam &s, 
	       const LcParam &df, const LcParam &ex)
    : dayMax(&dmax),  stretch(&s), dFlux (&df), ext(&ex) {}
  //BandParams(const LcParam &dmax, const LcParam &s, 
  //	     const LcParam &df)
  //  : dayMax(&dmax),  stretch(&s), dFlux (&df), ext(NULL) {}
  
  BandParams() : dayMax(NULL), stretch(NULL), dFlux(NULL), ext(NULL) {};
  
  double DayMax() const {return dayMax->val;}
  double Stretch() const {return stretch->val;}
  double DFlux() const {return dFlux->val;}
  double Ext() const {if(ext) return ext->val; else return 0.;}
  
  friend class Model2Fit;
  friend class LcTemplate;
  friend class SpectrumTemplate;
  
  //! need a polymorphic type
  virtual ~BandParams() {}
  
};

vector<string> FitParamNames(const string &FileName);

#endif /* PARAM__H */
