// -*- C++ -*-
// 
// 
// 
//. 
#ifndef SALTMODEL_H
#define SALTMODEL_H

//#define WITH_HOST_EXTINCTION

#include <string>
#include <map>

#include "param.h"
#include "model2fit.h"
#include "sampledfunction.h"
#include "measureddata.h"
#include "corrector.h"





class SaltModel : public Model2Fit {

private:
  
  bool forcepositiveEBV;
  double GetMag(const Filter &filer, double phase, const string &magsys);
  
  //! reference wavelength for filter selection
  double ReferenceWavelength;
  
  //! parameters of this model
  LcParam *pDayMax;
  LcParam *pStretch;
  LcParam *pDflux;
  LcParam *pColor;
#ifdef WITH_HOST_EXTINCTION
  LcParam *pRv;
  LcParam *pTau;
#endif
  LcParam *pRedshift;


  class SnSpectra;
  //! the spectral template
  SnSpectra *SpectralTemplate;
  
  //! the correction polynomes   
  Corrector *polynomes;
  
  
public:
  SaltModel();
  virtual ~SaltModel() {};
  
  
  virtual double Flux(const Filter* filter,
		      const double observerday) const;

  virtual double SpectrumFlux(const double observerlambda, 
			      const double observerday) const;
  
  
  virtual string ModelName() const { return "SaltModel"; }
  
  
  virtual bool CanComputeFluxFor(const Filter &observerfilter,bool quiet=false) const;
    
  //virtual void KeepOnlyNBands(int n, MeasuredData* data);
  /*
  virtual double GetRestFrameMagAtPhase(const Filter& filter,
				double phase,
				string const& magsys="VEGA");
  
  virtual LcParam GetMagAtPhase(const Filter& filter,
				double phase,
				string const& magsys="VEGA");
  
  */
  virtual void InitParams(const MeasuredData* data);
  
  
  
  double GetPhase(double day) const;
  
  void ForcePositiveEBV(bool yes = "true") { forcepositiveEBV=yes; } 

  const SnSpectra* GetSpectralTemplate() const {return SpectralTemplate;}

  
private:
  double FluxWithinPhaseRange(const Filter* MeasuredPassBand,
			      const double &Phase,
			      const double &RF_lambda) const;

  double SpectrumFluxWithinPhaseRange(const double &Phase,
						const double &RF_lambda) const;



  class SnSpectra : public Grid2DFunction {
  public: 
  
    SnSpectra(const string &FileName) : Grid2DFunction(FileName) {}

    bool CheckRange4Filter(const Filter& filter,double z) const;

    //!-

    double MinDate() const {return Xmin();}
    double MaxDate() const {return Xmax();}

  }; // end of class SnSpectra



 
};



// create minimal number of parameters needed for the fit
//void CreateParams(); // do we need this ???
//  map<const string,BandParams*> & GetParams4Band() {return Params4Band;};
// dynamic association of ObserverFilters to parameters
//  BandParams* GetParams4ObserverFilter(const Filter* filter, bool computeifnecessary=true);
//  void FitSingleBand(const string &band);  
//  LcParam GetRestFrameMagAtPhase(double phase, const string &magsys= "VEGA") ;
//  LcParam GetUnextinctedRestFrameMagAtPhase(double phase, const string &magsys= "VEGA") ;
//  const TemplateCurves& GetTemplateCurves() const {return templateCurves;}  
//  void SetRedshift(double zz) {z=zz;BandOfFit4ObserverFilter.clear();Params4ObserverFilter.clear();};  


#endif /* MODEL2FIT__H */
