
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>

#include "measureddata.h"
#include "saltmodel.h"
#include "salt2model.h"
#include "fileutils.h"
#include "dictfile.h"
#include "magsys.h"



static void usage(const char *pg)
{ 
  std::cerr << pg << " dumps peak observed magnitude" << endl;
  std::cerr << "usage : " << pg << " <lightcurve> <parameter_filename>" << std::endl;
  std::cerr << " <parameter_filename> is the path of the output file of snfit: (i.e. result_salt.dat or result_salt2.dat)" << endl; 
  exit(EXIT_FAILURE);  
}

int main(int nargs, char **args) {
  if (nargs !=3 ) usage(args[0]);
  
  std::vector<std::string> lc;
  lc.push_back(args[1]);
  string parameters_filename = args[2];
  
  MeasuredData md(lc,false);
  DictFile file(lc[0]);
  string magsys = file.GlobalValue("MAGSYS");
    
  
  
  // read model name
  string model_name = "";
  {
    std::ifstream stream(parameters_filename.c_str());
    if(!stream) {
      cerr << "error: cannot read paramete file " << parameters_filename << endl;
      usage(args[0]);
    }
    
    if(!(stream >> model_name)) {
      cerr << "error: cannot read model name in " << parameters_filename << endl;
      exit(EXIT_FAILURE);
    }
    stream.close();
  }
  Model2Fit *model=0;
  if(model_name=="SaltModel")
    model = new SaltModel();
  if(model_name=="Salt2Model")
    model = new Salt2Model();
  if(!model) {
    cerr << "error: cannot deal with model '" << model_name << "'" << endl;
    exit(-1);
  }
  model->SetWavelengthRange(0.,100000.);
  model->ReadParams(parameters_filename);
  
  // for salt1 backward compatibility
  if(model->params.LocateParam("Redshift")->err<0) {
    cout << "warning : redshift parameter is not set in file, set it to " << md.Zhelio() << endl;
    model->SetParameterValue("Redshift",md.Zhelio());
  }
  
  
  
  //===============================================
  
  
  
  
  //double mag   = model->GetMagAtPhase(md.PointList(0).GetEffectiveFilter(),0,magsys);
  double zp   = GetMagSys("VEGA")->ZeroPoint(md.PointList(0).GetEffectiveFilter());
  double day  = model->params.LocateParam("DayMax")->val;
  double flux = model->Flux(&md.PointList(0).GetEffectiveFilterWithMilkyWay(),day);
  double mag  = -2.5*log10(flux)+zp;
  cout << mag << endl;
  
  return EXIT_SUCCESS;
}
