#include <iostream>

#include "matvect.h"
#include "sparsevect.h"



SparseVect vect_product(const SparseVect& A, const Vect& B) {  
  SparseVect res = A;
  const double* b =  B.Data();
  SparseVectIterator end = res.end();
  for(SparseVectIterator it = res.begin(); it != end; it++) {
    it->second *= b[it->first];
  }
  return res;
}





/*
==================================================================
 returns SV^t M V2
 this is the routine that slows down the fit
 assumes M is symmetric
==================================================================
*/
double scalar_product(const SparseVect& A, const Mat& B, const Vect& C) {

  size_t size = B.SizeX();
  double res=0;
  const double* b =  B.Data();
  SparseVectConstIterator a_end = A.end();
  if(size==1) {
    const double* c =  C.Data();
    for(SparseVectConstIterator a = A.begin(); a != a_end; a++) {
      res += a->second * b[a->first] * c[a->first];
    }
    return res;
  }
  double val;
  for(SparseVectConstIterator a = A.begin(); a != a_end; a++) {
    const double* by = b + size*a->first;
    const double* c = C.Data();
    val = 0;
    for(size_t i=0;i<size;++i,++by,++c)
      val += (*by)*(*c);
    res += a->second * val;
  }
  return res;
}

double scalar_product(const SparseVect& A, const Mat& B, const SparseVect& C) {

  size_t sizex = B.SizeX();
  double res=0;
  SparseVectConstIterator a_end = A.end();
  if(sizex==1) {
    const double* b =  B.Data();
    
    for(SparseVectConstIterator a = A.begin(); a != a_end; a++) {
      res += a->second * b[a->first] * C.get(a->first);
    }
    return res;
  }
  
  const double* b = B.Data();
  double val;
  size_t pos;
  for(SparseVectConstIterator a = A.begin(); a != a_end; a ++) {
    val = 0;
    pos = sizex*a->first;
    for(SparseVectConstIterator c = C.begin(); c != C.end(); c ++) {
      val += b[pos + c->first]*c->second;
    }
    res += a->second * val;
  }
  return res;
}

SparseVect vect_product(const Mat &A,const SparseVect& B) { 
  SparseVect AB;
  const double* a =  A.Data();
  SparseVectConstIterator end = B.end();
  size_t asize = A.SizeX();
  
  if(asize==1) { // diagonal matrix
    AB = B;
    SparseVectIterator end = AB.end();
    for(SparseVectIterator ab = AB.begin(); ab != end; ab++) {
      ab->second *= a[ab->first];
    }
  }else{
    size_t pos = 0;
    for(size_t j=0;j<asize;j++,pos+=asize) {
      double res = 0;
      for(SparseVectConstIterator b = B.begin(); b != end; b++) {
	res += a[b->first+pos]*b->second;
      }
      if(res)
	AB[j]=res;
    }
  }
  return AB;
}



double scalar_product(const SparseVect& V1,const SparseVect& V2) {
  SparseVectConstIterator it1 = V1.begin();
  SparseVectConstIterator it2 = V2.begin();
  std::map<size_t, double>::const_iterator end1 = V1.end();
  std::map<size_t, double>::const_iterator end2 = V2.end();
  
  double res = 0;
  
  while(it1!=end1 && it2!=end2) {
    if( it1->first == it2->first) {
      res += it1->second * it2->second;
      it1++;
      it2++;
    }else{
      if(it1->first<it2->first) it1++;
      else it2++;
    }
  }
  
  return res;
}

double scalar_product(const SparseVect& V1,const Vect& V2) {
  
  std::map<size_t, double>::const_iterator end1 = V1.end();
  
  double res = 0;
  const double * v2 = V2.Data();
  for(SparseVectConstIterator it1 = V1.begin(); it1 != end1; it1++) {
    res += it1->second * v2[it1->first];
  }
  return res;
}
