#ifndef SPARSEVECT__H
#define SPARSEVECT__H

#include <map>
#include <iostream>





class Vect;
class Mat;

class SparseVect : public std::map<size_t, double>
{
  
 public:

  SparseVect() {} 
  
  
  double get(const size_t i) const {
    const_iterator it = find(i);
    if (it == end()) 
      return 0;
    else
      return it->second;
  }
  // equivalent to operator [] of map, just for convinience with existing code
  double set(const size_t i, const double& val)  {
    (*this)[i]=val;
    return val;
  }
  
  int writeASCII(std::ostream& Stream) const {
     Stream << size() << std::endl;
     for(const_iterator it = begin(); it != end(); it++) {
       Stream << it->first << " " << it->second << std::endl;
     }
     return 0;
  }
  
  int readASCII(std::istream& Stream) {
    size_t n;
    Stream >> n;
    int index;
    double val;
    for(size_t i=0;i<n;i++) {
      Stream >> index >> val;
      (*this)[index]=val;
    }
    return 0;
  }
  
  friend std::ostream& operator << (std::ostream &stream, const SparseVect &v)
    { v.writeASCII(stream); return stream;}

};

typedef SparseVect::const_iterator SparseVectConstIterator;
typedef SparseVect::iterator SparseVectIterator;

SparseVect vect_product(const SparseVect& SV, const Vect& V);
SparseVect vect_product(const Mat &W,const SparseVect& V);
double scalar_product(const SparseVect& SV, const Mat& M, const Vect& V2);
double scalar_product(const SparseVect& SV, const Mat& M, const SparseVect& SV2);
double scalar_product(const SparseVect& V1,const SparseVect& V2);
double scalar_product(const SparseVect& V1,const Vect& V2);


#endif /* SPARSEVECT__H */
