#ifndef SPECTRUMFIT__H
#define SPECTRUMFIT__H

#include <iostream>
#include <string>
#include <map>
#include <vector>

#include "measureddata.h"
#include "model2fit.h"
#include "param.h"



/*! a class for fitting light curve models to data with Minuit 
 *
 * all pertinent info is in measuredData and model (which is a virtual class)
 * contains all the minuit handling
 */
class SpectrumFit
{
  private :

    
    const MeasuredSpectrum &measuredSpectrum;
    Model2Fit &model;
    
    double chi2Val;
    double chi2Valpdf;
    double lambda_min;
    double lambda_max;
    
    public :
      
      SpectrumFit(const MeasuredSpectrum &Spectrum, Model2Fit& Model);
    
    Model2Fit* GetModel() {return &model;};
    
    
    
    int NPar () const {return model.params.size();}
    
    double Chi2() const;
    
    void InitFit();
    
    void SetWavelengthRange(const double& lmin, const double& lmax);

    bool Minimize();
    
    void dump(std::ostream & stream = std::cout) const{
      stream << model.ModelName() << endl;
      model.params.dump(stream);
    }
    friend ostream& operator << (ostream &stream, const SpectrumFit &s)
      { s.dump(stream); return stream;}
    

    virtual ~SpectrumFit() {}


};



#endif /* SPECTRUMFIT__H */
