#ifndef TEMPLATECURVES__H
#define TEMPLATECURVES__H


#include <string>
#include <map>

#include "instrument.h"
#include "snspectra.h"
#include "sampledfunction.h"
#include "legsplit.h"

// Rv value used in the model
#define RV_IN_FIT 1.0 // was 1.

//! storage of template lightcurves and associated bandpasses.
class TemplateCurves
{
  FilterWheel bands; // the bands for the lightcurves
  typedef  map<string, General1DFunction> LcMap;
  typedef LcMap::const_iterator LcMapCIterator;
  /* the map holds the lightcurves , and adresses them by 
     the filter name (in the FilterWheel)
  */
  void init(const string &ModelFileName);

  LcMap lightCurves; 
  SnSpectra *templateSpectrum;
  LegSplit3 *polynomes;

  public :
    
    TemplateCurves();
    TemplateCurves(const string &ModelFileName);
    
  const FilterWheel& Bands() const { return bands;}
  const General1DFunction &LightCurve(const string &BandName) const;
  const Filter &GetFilter(const string &BandName) const 
    {return bands.GetFilterByBand(BandName);}
  const SnSpectra& TemplateSpectrum() const {return *templateSpectrum;}
  
  double ExpectedFlux(const string &FitBandName, 
		      const double z,
		      const General1DFunction &MeasuredPassBand,
		      const double phase, const double Stretch = 1, const double Ext = 0) const;

  double spectrumFlux(const General1DFunction &MeasuredPassBand,
		      const double z,
		      const double phase) const;
};



#endif /* TEMPLATECURVES__H */
