
#include "wmap5cosmo.h"
#include "fatalerror.h"

#include <cmath>
#include <iostream>


using namespace std;

static double sq(const double &x) { return x*x;}
static double cub(const double &x) { return x*x*x;}

double Wmap5Cosmo::zstar() const
{
  double omegamh2 = omegam*sq(h0/100);
  double g1=0.0783*pow(obh2,-0.238)/(1.+ 39.5*pow(obh2,0.763));
  double g2 = 0.560/(1+21.1*pow(obh2,1.81));
  return 1048*(1.+0.00124*pow(obh2,-0.738))*(1+g1*pow(omegamh2,g2));
}


double Wmap5Cosmo::integ_hinv(const double &Zmin, const double &Zmax, 
			      const int NStep) const
{
  // changement de variable u = 1./sqrt(1+z)
  double umax = 1./sqrt(1+Zmin);
  double umin = 1./sqrt(1+Zmax);
  double step = (umax-umin)/NStep;
  double u = umin+0.5*step;
  double sum =0;
  for (int i=0; i<NStep; ++i, u+=step)
    {
      double u2 = sq(u);
      double u3hinv = sqrt(omegam + omegax*pow(u,-6*w) + omegak*u2 + omegar/u2);
      sum += 1./u3hinv;
    }
  return 2.*sum*step;;
}


double Wmap5Cosmo::rs(const double Z) const
{// eq 6,7,8
  double amax= 1/(1+Z);
  double amin=0;
  int nstep = 100;
  double step = (amax-amin)/nstep;
  double a=amin+0.5*step;
  double sum = 0;
  double density_ratio = 3*obh2/sq(h0/100)/(4.*omegar/(1+0.2271*3.04));
  for (int i=0; i<nstep ; ++i , a+= step)
    {
      double a4h2 = (omegam+omegak*a)*a+omegar+omegax*pow(a,4-3.*w);
      double fact = 1.+density_ratio*a;
      sum += 1./sqrt(a4h2*fact);
    }
  return sum*step/sqrt(3.);
}
  
double Wmap5Cosmo::l_a(const double Z,const int NStep) const
{
  return M_PI*Dm(Z,NStep)/rs(Z);
}

double Wmap5Cosmo::R(const double Z,const int NStep) const  
{
  return sqrt(omegam)*Dm(Z,NStep);
}

double Wmap5Cosmo::E(const double z) const
{
  return sqrt(omegam*cub(1.+z) + 
	      omegax*pow(1+z,3*(1+w))
	      + sq(1+z)*(omegak+sq(1+z)*omegar));
}

void  Wmap5Cosmo::SetParam(int I, double Val)
{
  /* omegam, omegax, w,  */
  if (I==0) *this=Wmap5Cosmo(Val, omegax, w, obh2,h0);
  else if (I==1) *this=Wmap5Cosmo(omegam, Val, w, obh2, h0);
  else if (I==2) *this=Wmap5Cosmo(omegam, omegax, Val, obh2,h0);
  else if (I==3) *this=Wmap5Cosmo(omegam, omegax, w, Val, h0);
  else if (I==4) *this=Wmap5Cosmo(omegam, omegax, w, obh2, Val);
  else FatalError(" wrong param rank in Wmap5Cosmo::SetParam" );
}

double  Wmap5Cosmo::GetParam(int I) const
{
  if (I==0) return omegam;
  else if (I==1) return omegax;
  else if (I==2) return w;
  else if (I==3) return obh2;
  else if (I==4) return h0;
  else FatalError(" wrong param rank in Wmap5Cosmo::GetParam" );
  return -1;
}

#include <cstring> // for strcmp
#include <cstdlib> // for abort

int Wmap5Cosmo::GetParamRank(const char *Name) const
{
  static const char *names[5]={"Omegam", "Omegal", "w0", "OmegabH2", "H0"};
  for (int k=0; k<5; ++k)
    if (strcmp(Name,names[k]) ==0) return k;
  FatalError("Wmap5Cosmo::GetParamRank,unknown param : "+string(Name));
  abort();
}


void Wmap5Cosmo::dump(std::ostream& S) const
{
  S << " Wmap5Cosmo om-ox-w-h0 " 
    << omegam << ' ' << omegax << ' ' << w << ' ' << h0;
}

  
