#ifndef WMAP5COSMO__H
#define WMAP5COSMO__H

#include "iostream"
#include "cosmology.h"

class Wmap5Cosmo : public Cosmology
{
  double omegam;
  double omegax;
  double w;
  double obh2;
  double h0;
  double omegak;
  double omegar;

  /* Intended to compute only distance constraints.
     Distances are dimension-less (c=H0=1)
  */
     


public:
  Wmap5Cosmo(const double Omegam, const double Omegax, const double W, const double OmegabH2, const double H0) : 
    Cosmology(1-Omegam-Omegax) , omegam(Omegam), omegax(Omegax), w(W), obh2(OmegabH2), h0(H0)
  {
    omegak = 1-omegam-omegax;
    omegar = 2.469e-5*(1+0.2271/* extra contribution from nu's */ *3.04 /* N_nu */)/(h0*h0/10000);
  }


  double Omegam() const { return omegam;}

  int NPar() const { return 4;}

  /* zstar */
  double zstar() const;

  /* comoving sound horizon */
  double rs(const double Z) const;


  double l_a(const double Z, const int NStep=100) const;

  double R(const double Z, const int NStep=100) const;

  double integ_hinv(const double &Zmin, const double &Zmax, 
		    const int NStep) const;

  double E(const double z) const;

  void dump(std::ostream&) const;

  virtual Cosmology* Clone() const
  {
    return new Wmap5Cosmo(omegam,omegax,w,obh2,h0);
  }

  void SetParam(int, double);
  double GetParam(int) const;

  int GetParamRank(const char *Name) const;


};


#endif
